@@ -34,6 +34,7 @@ from thoth.common import RuntimeEnvironment
 from thoth.python import Project
 
 from .units.boots import Boot1
+from .units.pseudonyms import Pseudonym1
 from .units.sieves import Sieve1
 from .units.steps import Step1
 from .units.strides import Stride1
@@ -76,12 +77,27 @@ def pipeline_config() -> PipelineConfig:
     flexmock(PipelineConfig)
 
     flexmock(Boot1)
+    flexmock(Pseudonym1)
     flexmock(Sieve1)
     flexmock(Step1)
     flexmock(Stride1)
     flexmock(Wrap1)
 
-    return PipelineConfig(boots=[Boot1()], sieves=[Sieve1()], steps=[Step1()], strides=[Stride1()], wraps=[Wrap1()],)
+    boot1 = Boot1()
+    pseudonym1 = Pseudonym1()
+    sieve1 = Sieve1()
+    step1 = Step1()
+    stride1 = Stride1()
+    wrap1 = Wrap1()
+
+    return PipelineConfig(
+        boots={boot1.configuration.get("package_name"): [boot1]},
+        pseudonyms={pseudonym1.configuration["package_name"]: [pseudonym1]},
+        sieves={sieve1.configuration.get("package_name"): [sieve1]},
+        steps={step1.configuration.get("package_name"): [step1]},
+        strides={stride1.configuration.get("package_name"): [stride1]},
+        wraps={wrap1.configuration.get("package_name"): [wrap1]},
+    )
 
 
 @pytest.fixture
