@@ -18,8 +18,9 @@
 """Implementation of Adaptive Simulated Annealing (ASA) used to resolve software stacks."""
 
 from typing import Any
-from typing import Tuple
 from typing import List
+from typing import Optional
+from typing import Tuple
 import logging
 import random
 import math
@@ -42,13 +43,17 @@ _LOGGER = logging.getLogger(__name__)
 class AdaptiveSimulatedAnnealing(Predictor):
     """Implementation of adaptive simulated annealing looking for stacks based on the scoring function."""
 
-    _temperature_history = attr.ib(type=List[Tuple[float, bool, float, int]], factory=list, kw_only=True,)
+    temperature_coefficient = attr.ib(type=float, default=0.999, kw_only=True)
+
+    _temperature_history = attr.ib(
+        type=List[Tuple[Optional[float], Optional[bool], Optional[float], int]], factory=list, kw_only=True,
+    )
     _temperature = attr.ib(type=float, kw_only=True, default=0.0)
 
     def _temperature_function(self, t0: float, context: Context) -> float:
         """Temperature function used to compute new temperature."""
         k = (context.accepted_final_states_count + math.log(context.iteration + 1)) / context.limit
-        temperature = t0 * 0.97 ** k
+        temperature = t0 * self.temperature_coefficient ** k
         _LOGGER.debug(
             "New temperature for (iteration=%d, t0=%g, accepted final states=%d, limit=%d, beam size= %d, k=%f) = %g",
             context.iteration,
