@@ -35,6 +35,9 @@ import contextlib
 import signal
 import weakref
 
+import matplotlib
+import matplotlib.pyplot as plt
+from matplotlib.font_manager import FontProperties
 from thoth.common import get_justification_link as jl
 from thoth.python import PackageVersion
 from thoth.python import Project
@@ -49,6 +52,7 @@ from .exceptions import BootError
 from .exceptions import CannotProduceStack
 from .exceptions import EagerStopPipeline
 from .exceptions import NotAcceptable
+from .exceptions import NoHistoryKept
 from .exceptions import SkipPackage
 from .exceptions import SieveError
 from .exceptions import StepError
@@ -73,10 +77,6 @@ _LOGGER = logging.getLogger(__name__)
 _NO_EXTRAS = frozenset([None])
 
 
-class _NoStateAdd(Exception):
-    """An exception used internally to signalize no state addition."""
-
-
 def _beam_width(value: Any) -> Optional[int]:
     """Set and convert beam width."""
     if value is None:
@@ -149,8 +149,7 @@ class Resolver:
     DEFAULT_COUNT = 3
     DEFAULT_BEAM_WIDTH = -1
     DEFAULT_LIMIT_LATEST_VERSIONS = -1
-    DEFAULT_LOG_FINAL_STATE_COUNT = 500
-    DEFAULT_LOG_FINAL_STATE_TOP = False
+
     pipeline = attr.ib(type=PipelineConfig, kw_only=True)
     project = attr.ib(type=Project, kw_only=True)
     library_usage = attr.ib(type=Dict[str, Any], kw_only=True, converter=_library_usage)
@@ -172,11 +171,13 @@ class Resolver:
 
     cli_parameters = attr.ib(type=Dict[str, Any], default=attr.Factory(dict), kw_only=True)
     stop_resolving = attr.ib(type=bool, default=False, kw_only=True)
-    log_final_state_count = attr.ib(type=int, kw_only=True)
+    log_iteration = attr.ib(type=int, kw_only=True, default=int(os.getenv("THOTH_ADVISER_LOG_ITERATION", 7500)))
 
     _beam = attr.ib(type=Optional[Beam], kw_only=True, default=None)
     _solver = attr.ib(type=Optional[PythonPackageGraphSolver], kw_only=True, default=None)
     _context = attr.ib(type=Optional[Context], default=None, kw_only=True)
+    _history = attr.ib(type=List[Optional[float]], factory=list, init=False)
+    _history_max = attr.ib(type=List[Optional[float]], factory=list, init=False)
 
     _log_unresolved = attr.ib(type=Set[Tuple[str, str, str]], default=attr.Factory(set), kw_only=True)
     _log_unsolved = attr.ib(type=Set[str], default=attr.Factory(set), kw_only=True)
@@ -195,17 +196,11 @@ class Resolver:
         if value <= 0:
             raise ValueError(f"Value for attribute {attribute!r} should be a positive integer, got {value} instead")
 
-    @log_final_state_count.default
-    def _log_final_state_count_default(self) -> int:
-        """Determine how often the pipeline should log about its progress."""
-        # Log each 10% by default.
-        return int(os.getenv("THOTH_ADVISER_LOG_FINAL_STATE_COUNT", self.limit // 10)) or 1
-
     @property
     def context(self) -> Context:
         """Retrieve context bound to the current resolver."""
         if self._context is None:
-            raise ValueError("Context not assigned yes to the current resolver instance")
+            raise ValueError("Context not assigned yet to the current resolver instance")
 
         return self._context
 
@@ -249,14 +244,23 @@ class Resolver:
             cli_parameters=self.cli_parameters,
         )
 
-    def _run_boots(self) -> None:
-        """Run all boots bound to the current annealing run context."""
-        for boot in self.pipeline.boots:
+    def _run_boots(self, *, with_devel: bool = True) -> None:
+        """Run all boots bound to the current run context."""
+        package_boots = []
+        for package_version in self.project.iter_dependencies(with_devel=with_devel):
+            package_boots.extend(self.pipeline.boots_dict.get(package_version.name, []))
+
+        for boot in chain(package_boots, self.pipeline.boots_dict.get(None, [])):
             _LOGGER.debug("Running boot %r", boot.__class__.__name__)
+            boot.unit_run = True
             try:
                 boot.run()
             except NotAcceptable as exc:
-                raise CannotProduceStack(f"Boot pipeline unit {boot.__class__.__name__} failed: {str(exc)!r}")
+                msg = f"Boot pipeline unit {boot.__class__.__name__!r} failed: {str(exc)}"
+                self.context.stack_info.append(
+                    {"message": msg, "type": "ERROR",}
+                )
+                raise CannotProduceStack(msg, stack_info=self.context.stack_info)
             except Exception as exc:
                 raise BootError(f"Failed to run pipeline boot {boot.__class__.__name__!r}: {str(exc)}") from exc
 
@@ -265,23 +269,27 @@ class Resolver:
     ) -> Generator[PackageVersion, None, None]:
         """Run sieves on each package tuple."""
         result = (pv for pv in package_versions)
-        for sieve in self.pipeline.sieves:
-            _LOGGER.debug("Running sieve %r", sieve.__class__.__name__)
-            try:
-                result = sieve.run(result)
-            except SkipPackage:
-                raise
-            except NotAcceptable as exc:
-                _LOGGER.log(
-                    log_level, "Sieve %r removed packages %r: %s", sieve.__class__.__name__, str(exc),
-                )
-                result = []  # type: ignore
-                break
-            except Exception as exc:
-                raise SieveError(
-                    f"Failed to run sieve {sieve.__class__.__name__!r} for "
-                    f"Python packages {[pv.to_tuple() for pv in package_versions]}: {str(exc)}"
-                ) from exc
+        if package_versions:
+            for sieve in chain(
+                self.pipeline.sieves_dict.get(package_versions[0].name, []), self.pipeline.sieves_dict.get(None, [])
+            ):
+                _LOGGER.debug("Running sieve %r", sieve.__class__.__name__)
+                sieve.unit_run = True
+                try:
+                    result = sieve.run(result)
+                except SkipPackage:
+                    raise
+                except NotAcceptable as exc:
+                    _LOGGER.log(
+                        log_level, "Sieve %r removed packages %r: %s", sieve.__class__.__name__, str(exc),
+                    )
+                    result = []  # type: ignore
+                    break
+                except Exception as exc:
+                    raise SieveError(
+                        f"Failed to run sieve {sieve.__class__.__name__!r} for "
+                        f"Python packages {[pv.to_tuple() for pv in package_versions]}: {str(exc)}"
+                    ) from exc
 
         yield from result
 
@@ -290,6 +298,7 @@ class Resolver:
         state: State,
         package_version: PackageVersion,
         unresolved_dependencies: Optional[Dict[str, List[Tuple[str, str, str]]]] = None,
+        newly_added: Optional[List[Tuple[str, str, str]]] = None,
         *,
         user_stack_scoring: bool = False,
         log_level: int = logging.DEBUG,
@@ -317,13 +326,16 @@ class Resolver:
         score_addition = 0.0
         justification_addition = []
         skip_package = False
-        for step in self.pipeline.steps:
+        for step in chain(
+            self.pipeline.steps_dict.get(package_version.name, []), self.pipeline.steps_dict.get(None, [])
+        ):
             _LOGGER.debug("Running step %r for %r", step.__class__.__name__, package_version_tuple)
+            step.unit_run = True
 
-            if multi_package_resolution and not step.MULTI_PACKAGE_RESOLUTIONS:
+            if multi_package_resolution and not step.configuration["multi_package_resolution"]:
                 _LOGGER.debug(
                     "Skipping running step %r - this step was already run for package %r "
-                    "and the given step has no MULTI_PACKAGE_RESOLUTIONS flag set",
+                    "and the given step has no multi_package_resolution flag set",
                     step.__class__.__name__,
                     package_version_tuple,
                 )
@@ -332,7 +344,7 @@ class Resolver:
             try:
                 step_result = step.run(state, package_version)
             except SkipPackage as exc:
-                # This should be fine also for user-stacks steps. They will just adjusted.
+                # This should be fine also for user-stacks steps. The recommendation engine will compute alternatives.
                 log_once(
                     _LOGGER,
                     self._log_step_skip_package,
@@ -410,15 +422,15 @@ class Resolver:
         else:
             # Optimization - reuse the old one as it would be discarded anyway.
             cloned_state = state
-            if (
-                not user_stack_scoring
-                and not skip_package
-                and (score_addition != 0.0 or (not state.unresolved_dependencies and not unresolved_dependencies))
+            if not user_stack_scoring and (
+                (not skip_package and score_addition != 0.0)
+                or (not state.unresolved_dependencies and not unresolved_dependencies)
             ):
                 self.beam.remove(cloned_state)
 
         if unresolved_dependencies and not skip_package:
             cloned_state.set_unresolved_dependencies(unresolved_dependencies)
+            self._run_pseudonyms(cloned_state, newly_added)
 
         cloned_state.remove_unresolved_dependency_subtree(package_version_tuple[0])
         if not skip_package:
@@ -440,8 +452,13 @@ class Resolver:
 
     def _run_strides(self, state: State) -> bool:
         """Run strides and check if the given state should be accepted."""
-        for stride in self.pipeline.strides:
+        package_strides = []
+        for package_name in state.resolved_dependencies:
+            package_strides.extend(self.pipeline.strides_dict.get(package_name, []))
+
+        for stride in chain(package_strides, self.pipeline.strides_dict.get(None, [])):
             _LOGGER.debug("Running stride %r", stride.__class__.__name__)
+            stride.unit_run = True
             try:
                 stride.run(state)
             except NotAcceptable as exc:
@@ -455,13 +472,18 @@ class Resolver:
         return True
 
     def _run_wraps(self, state: State) -> None:
-        """Run all wraps bound to the current annealing run context."""
-        for wrap in self.pipeline.wraps:
+        """Run all wraps bound to the current run context."""
+        package_wraps = []
+        for package_name in state.resolved_dependencies:
+            package_wraps.extend(self.pipeline.wraps_dict.get(package_name, []))
+
+        for wrap in chain(package_wraps, self.pipeline.wraps_dict.get(None, [])):
             _LOGGER.debug("Running wrap %r", wrap.__class__.__name__)
+            wrap.unit_run = True
             try:
                 wrap.run(state)
             except Exception as exc:
-                raise WrapError(f"Failed to run wrap {wrap.__class__.__name__!r} on a final step: {str(exc)}") from exc
+                raise WrapError(f"Failed to run wrap {wrap.__class__.__name__!r} on a final state: {str(exc)}") from exc
 
     def _prepare_user_lock_file(self, *, with_devel: bool = True) -> None:
         """Perform operations on the user's lock file required before running the pipeline.
@@ -606,10 +628,7 @@ class Resolver:
                 # database but none was matching the given version range.
                 unresolved.append(package_version.name)
 
-                error_msg = (
-                    f"No versions were found for direct dependency "
-                    f"{package_version.name!r} - see {jl('solve_direct')}"
-                )
+                error_msg = f"No versions were found for direct dependency {package_version.name!r}"
                 runtime_environment = self.project.runtime_environment
                 if runtime_environment.operating_system.name:
                     error_msg += f"; operating system {runtime_environment.operating_system.name!r}"
@@ -622,7 +641,10 @@ class Resolver:
                 if runtime_environment.platform:
                     error_msg += f" using platform {runtime_environment.platform!r}"
 
-                _LOGGER.warning(error_msg)
+                self.context.stack_info.append(
+                    {"message": error_msg, "type": "ERROR", "link": jl("solve_direct"),}
+                )
+                _LOGGER.warning("%s - see %s", error_msg, jl("solve_direct"))
                 continue
 
             _LOGGER.info(
@@ -632,20 +654,28 @@ class Resolver:
             )
 
         if unresolved:
-            _LOGGER.warning("Resolver failed as it was unable to resolve direct dependencies, see %s", jl("unresolved"))
-            raise UnresolvedDependencies("Unable to resolve all direct dependencies", unresolved=unresolved)
+            error_msg = "Resolver failed as it was unable to resolve direct dependencies"
+            self.context.stack_info.append({"message": error_msg, "type": "ERROR", "link": jl("unresolved")})
+            _LOGGER.warning("%s - see %s", error_msg, jl("unresolved"))
+            raise UnresolvedDependencies(
+                "Unable to resolve all direct dependencies", unresolved=unresolved, stack_info=self.context.stack_info
+            )
 
-        # Now we are free to de-instantiate solver.
-        del self._solver
+        # Now we are free to de-instantiate solver to save some memory.
+        self._solver = None
 
         return resolved_direct_dependencies
 
-    def _prepare_initial_state(self, *, with_devel: bool) -> None:
+    def _prepare_initial_state(self, *, with_devel: bool) -> State:
         """Prepare initial state for resolver."""
         direct_dependencies = self._resolve_direct_dependencies(with_devel=with_devel)
 
         if not direct_dependencies:
-            raise CannotProduceStack("No direct dependencies found")
+            msg = "No direct dependencies found"
+            self.context.stack_info.append(
+                {"message": msg, "type": "ERROR", "link": jl("solve_direct"),}
+            )
+            raise CannotProduceStack(msg, stack_info=self.context.stack_info)
 
         skipped_packages: List[str] = []
         for direct_dependency_name, package_versions in direct_dependencies.items():
@@ -662,10 +692,14 @@ class Resolver:
                 continue
 
             if not package_versions:
-                raise CannotProduceStack(
-                    f"Cannot satisfy direct dependencies - direct dependencies "
-                    f"of type {direct_dependency_name!r} were removed by pipeline sieves"
+                msg = (
+                    f"Cannot satisfy direct dependencies - direct dependencies of type {direct_dependency_name!r} "
+                    "were removed by pipeline sieves"
                 )
+                self.context.stack_info.append(
+                    {"type": "ERROR", "message": msg,}
+                )
+                raise CannotProduceStack(msg, stack_info=self.context.stack_info)
 
             if self.limit_latest_versions:
                 direct_dependencies[direct_dependency_name] = package_versions[: self.limit_latest_versions]
@@ -684,6 +718,56 @@ class Resolver:
         state = State.from_direct_dependencies(direct_dependencies)
         weakref.finalize(state, self.predictor.finalize_state, id(state)).atexit = False
         self.beam.add_state(state)
+        return state
+
+    def _run_pseudonyms(self, state: State, package_tuples: Optional[List[Tuple[str, str, str]]] = None) -> None:
+        """Run pseudonyms for the given package, clone state and add it to beam if needed."""
+        for package_tuple in package_tuples or []:
+            self._run_pseudonym_units(state, package_tuple)
+
+    def _run_pseudonym_units(self, state: State, package_tuple: Tuple[str, str, str]) -> None:
+        """Run pseudonym units the given package tuple."""
+        package_version: PackageVersion = self.context.get_package_version(package_tuple)
+        for unit in self.pipeline.pseudonyms_dict.get(package_tuple[0], []):
+            _LOGGER.debug("Running pseudonym %r", unit.__class__.__name__)
+            unit.unit_run = True
+            for pseudonym_package_tuple in unit.run(package_version):
+                # Pseudonyms introduced do not have dependents (we work on direct dependencies - initial state).
+                if pseudonym_package_tuple[0] in state.resolved_dependencies:
+                    _LOGGER.warning(
+                        "Pseudonym %r already present in a resolved form, skipping pseudonym creation",
+                        pseudonym_package_tuple,
+                    )
+                    continue
+
+                if (
+                    pseudonym_package_tuple
+                    in state.unresolved_dependencies.get(pseudonym_package_tuple[0], {}).values()
+                ):
+                    _LOGGER.debug(
+                        "Pseudonym %r already present in state in unresolved form, not creating a new state",
+                        pseudonym_package_tuple,
+                    )
+                    continue
+
+                self.context.register_package_tuple(
+                    pseudonym_package_tuple,
+                    develop=package_version.develop,
+                    os_name=None,  # TODO: pass based on the package_tuple
+                    os_version=None,
+                    python_version=None,
+                )
+                new_state = state.clone()
+                new_state.remove_unresolved_dependency_subtree(package_tuple[0])
+                new_state.add_unresolved_dependency(pseudonym_package_tuple)
+                self.beam.add_state(new_state)
+
+    def _run_pseudonyms_initial_state(self, state: State) -> None:
+        """Run pseudonyms on an initial state."""
+        to_run_pseudonyms = state.unresolved_dependencies.keys() & self.pipeline.pseudonyms_dict.keys()
+        for pseudonym_name in to_run_pseudonyms:
+            for package_tuple in state.unresolved_dependencies[pseudonym_name].values():
+                self._run_pseudonym_units(state, package_tuple)
 
     def _expand_state(self, state: State, package_tuple: Tuple[str, str, str]) -> Optional[State]:
         """Expand the given state, generate new states respecting the pipeline configuration.
@@ -797,6 +881,7 @@ class Resolver:
 
         package_tuple = package_version.to_tuple()
         all_dependencies: Dict[str, List[Tuple[str, str, str]]] = {}
+        newly_added: List[Tuple[str, str, str]] = []
         for dependency_name, dependency_version in dependencies:
             records = self.graph.get_python_package_version_records(
                 package_name=dependency_name,
@@ -837,6 +922,7 @@ class Resolver:
                     os_version=record["os_version"],
                     python_version=record["python_version"],
                 )
+                newly_added.append(dependency_tuple)
 
                 if dependency_tuple not in all_dependencies[dependency_tuple[0]]:
                     all_dependencies[dependency_tuple[0]].append(dependency_tuple)
@@ -966,14 +1052,14 @@ class Resolver:
         for skipped_package in skipped_packages:
             all_dependencies.pop(skipped_package)
 
-        return self._run_steps(state, package_version, all_dependencies)
+        return self._run_steps(state, package_version, all_dependencies, newly_added)
 
-    def _do_resolve_states(
+    def _do_resolve_states_raw(
         self, *, with_devel: bool = True, user_stack_scoring: bool = True,
     ) -> Generator[State, None, None]:
         """Actually perform states resolution."""
         self._log_once_init()
-        self._run_boots()
+        self._run_boots(with_devel=with_devel)
 
         if not self.project.runtime_environment.is_fully_specified():
             _LOGGER.warning(
@@ -994,7 +1080,9 @@ class Resolver:
                 if user_stack:
                     yield user_stack
 
-        self._prepare_initial_state(with_devel=with_devel)
+        _LOGGER.info("Preparing initial states for the resolution pipeline")
+        state = self._prepare_initial_state(with_devel=with_devel)
+        self._run_pseudonyms_initial_state(state)
 
         _LOGGER.info("Hold tight, Thoth is computing recommendations for your application...")
 
@@ -1032,55 +1120,82 @@ class Resolver:
                 if state_returned is not None and not state_returned.unresolved_dependencies:
                     # A final state produced by the pipeline.
                     if self._run_strides(state_returned):
-                        self._run_wraps(state_returned)
                         self.context.accepted_final_states_count += 1
                         self.context.register_accepted_final_state(state_returned)
                         yield state_returned
                     else:
                         self.context.discarded_final_states_count += 1
 
+                    if self.beam.keep_history:
+                        self._history.append(state_returned.score)
+                        self._history_max.append(
+                            max(
+                                self._history_max[-1]
+                                if self._history_max and self._history_max[-1] is not None
+                                else state_returned.score,
+                                state_returned.score,
+                            )
+                        )
+                else:
+                    if self.beam.keep_history:
+                        self._history.append(None)
+                        self._history_max.append(self._history_max[-1] if self._history_max else None)
+
         if self.stop_resolving:
             _LOGGER.warning(
                 "Resolving stopped with the current beam size %d as the allocated CPU time was exhausted",
                 self.beam.size,
             )
 
-    def _do_resolve_products(
+    def _do_resolve_states(
         self, *, with_devel: bool = True, user_stack_scoring: bool = True,
-    ) -> Generator[Product, None, None]:
-        """Resolve raw products as produced by this resolver pipeline."""
+    ) -> Generator[State, None, None]:
+        """Resolve states as produced by this resolver pipeline, ready to be used to build a product."""
         if self.count > self.limit:
             _LOGGER.warning(
                 "Count (%d) is higher than limit (%d), setting count to %d", self.count, self.limit, self.limit,
             )
             self.count = self.limit
 
+        self._history.clear()
+        self._history_max.clear()
         self.predictor.pre_run()
         self.pipeline.call_pre_run()
 
         start_time = time.monotonic()
+        max_score = None
+        last_iteration_logged = 0
         try:
-            for final_state in self._do_resolve_states(with_devel=with_devel, user_stack_scoring=user_stack_scoring):
+            for final_state in self._do_resolve_states_raw(
+                with_devel=with_devel, user_stack_scoring=user_stack_scoring
+            ):
                 _LOGGER.debug(
-                    "Pipeline reached a new final state, yielding pipeline product " "with a score of %g - %d/%d",
+                    "Pipeline reached a new final state, yielding pipeline product with a score of %g - %d/%d",
                     final_state.score,
                     self.context.accepted_final_states_count,
                     self.context.limit,
                 )
 
-                if (self.context.accepted_final_states_count - 1) % self.log_final_state_count == 0:
+                max_score = final_state.score if max_score is None else max(max_score, final_state.score)
+
+                if (
+                    self.context.iteration - last_iteration_logged > self.log_iteration
+                    or self.context.accepted_final_states_count == 1
+                ):
                     _LOGGER.info(
                         "Pipeline reached %d final states out of %d requested in iteration %d "
-                        "(pipeline pace %.02f stacks/second), top rated software stack has a score of %s",
+                        "(pipeline pace %.02f stacks/second); top rated software stack in beam has a score of %.2f; "
+                        "top rated software stack found so far has a score of %.2f",
                         self.context.accepted_final_states_count,
                         self.context.limit,
                         self.context.iteration,
                         self.context.accepted_final_states_count / (time.monotonic() - start_time),
-                        self.beam.max().score if self.beam.size > 0 else "N/A",
+                        self.beam.max().score if self.beam.size > 0 else float("nan"),
+                        float("nan") if max_score is None else max_score,
                     )
+                    last_iteration_logged = self.context.iteration
 
-                product = Product.from_final_state(context=self.context, state=final_state)
-                yield product
+                yield final_state
                 del final_state
         except EagerStopPipeline as exc:
             _LOGGER.info("Stopping pipeline eagerly as per request: %s", str(exc))
@@ -1092,6 +1207,9 @@ class Resolver:
             self.context.accepted_final_states_count / duration,
         )
 
+        if max_score is not None:
+            _LOGGER.info("The highest rated software stack resolved has a score of %0.2f", max_score)
+
         _LOGGER.info(
             "Pipeline strides discarded %d and accepted %d final states in total",
             self.context.discarded_final_states_count,
@@ -1107,7 +1225,10 @@ class Resolver:
         """Resolve raw products as produced by this resolver pipeline."""
         self._init_context()
         with Unit.assigned_context(self.context), self.predictor.assigned_context(self.context):
-            yield from self._do_resolve_products(with_devel=with_devel, user_stack_scoring=user_stack_scoring)
+            for state in self._do_resolve_states(with_devel=with_devel, user_stack_scoring=user_stack_scoring):
+                # Always run wraps as raw products are computed.
+                self._run_wraps(state)
+                yield Product.from_final_state(context=self.context, state=state)
 
     def resolve(self, *, with_devel: bool = True, user_stack_scoring: bool = True) -> Report:
         """Resolve software stacks and return resolver report."""
@@ -1115,14 +1236,22 @@ class Resolver:
 
         self._init_context()
         with Unit.assigned_context(self.context), self.predictor.assigned_context(self.context):
-            for product in self._do_resolve_products(with_devel=with_devel, user_stack_scoring=user_stack_scoring):
-                report.add_product(product)
+            for state in self._do_resolve_states(with_devel=with_devel, user_stack_scoring=user_stack_scoring):
+                product = Product.from_final_state(context=self.context, state=state)
+                if report.add_product(product):
+                    # An optimization - we do not need to run wraps if the product is not part of the final result.
+                    self._run_wraps(state)
 
             if report.product_count() == 0:
-                raise CannotProduceStack(
-                    f"Resolver did not find any stack that would satisfy "
-                    f"requirements and stack characteristics given the time allocated - see {jl('no_stack')}"
+                msg = (
+                    f"Resolver did not find any stack that would satisfy requirements and stack "
+                    f"characteristics given the time allocated"
                 )
+                link = jl("no_stack")
+                self.context.stack_info.append(
+                    {"message": msg, "type": "ERROR", "link": link,}
+                )
+                raise CannotProduceStack(msg + f"- see {link}", stack_info=self.context.stack_info)
 
             if self.context.stack_info:
                 report.set_stack_info(self.context.stack_info)
@@ -1132,6 +1261,49 @@ class Resolver:
 
         return report
 
+    def plot(self) -> matplotlib.figure.Figure:
+        """Plot history captured during the resolution process."""
+        if not self._history:
+            raise NoHistoryKept("No history datapoints kept")
+
+        x = [i for i in range(len(self._history))]
+        y1 = self._history
+        y2 = self._history_max
+
+        fig, host = plt.subplots()
+        fig.subplots_adjust(right=0.75)
+
+        par1 = host.twinx()
+
+        par1.spines["right"].set_position(("axes", 1.10))
+        self.beam._make_patch_spines_invisible(par1)
+
+        par1.spines["right"].set_visible(True)
+        host.spines["right"].set_visible(False)
+        host.spines["top"].set_visible(False)
+
+        (p1,) = host.plot(x, y1, ",g", label="Score of the resolved stack")
+        (p2,) = par1.plot(x, y2, ",b", label="The highest score")
+
+        host.set_xlabel("iteration")
+        host.set_ylabel("Score of the resolved stack in iteration")
+        par1.set_ylabel("Score of the best resolved stack found")
+
+        host.yaxis.label.set_color(p1.get_color())
+        par1.yaxis.label.set_color(p2.get_color())
+
+        tkw = dict(size=4, width=1.5)
+        host.tick_params(axis="y", colors=p1.get_color(), **tkw)
+        host.tick_params(axis="x", **tkw)
+        par1.tick_params(axis="y", colors=p2.get_color(), **tkw)
+
+        font_prop = FontProperties()
+        font_prop.set_size("medium")
+        fig.legend(
+            loc="upper center", bbox_to_anchor=(0.50, 1.00), ncol=2, fancybox=True, shadow=True, prop=font_prop,
+        )
+        return fig
+
     @classmethod
     def get_adviser_instance(
         cls,
