@@ -29,10 +29,11 @@ from typing import Optional
 from typing import TYPE_CHECKING
 
 import attr
+from voluptuous import Required
+from voluptuous import Schema
 
 from ..state import State
 from ..stride import Stride
-from ..exceptions import StrideError
 from ..exceptions import NotAcceptable
 
 if TYPE_CHECKING:
@@ -45,12 +46,13 @@ _LOGGER = logging.getLogger(__name__)
 class OneVersionStride(Stride):
     """Filter out software stacks allowing the pipeline to produce just one software stack with specific package."""
 
-    version_seen = attr.ib(type=Optional[str], default=None)  # type: ignore
+    version_seen = attr.ib(type=Optional[str], default=None, init=False)
 
     CONFIGURATION_DEFAULT = {
         "package_name": None,
         "only_once": True,
     }
+    CONFIGURATION_SCHEMA = Schema({Required("package_name"): str, "only_once": bool})
 
     @classmethod
     def should_include(cls, builder_context: "PipelineBuilderContext") -> Optional[Dict[str, Any]]:
@@ -61,9 +63,6 @@ class OneVersionStride(Stride):
         """Initialize internal state of the unit."""
         self.version_seen = None
 
-        if self.configuration["package_name"] is None:
-            raise StrideError(f"No package name provided to the {self.__class__.__name__!r} pipeline unit")
-
     def run(self, state: State) -> None:
         """Filter out software stacks, allow only a package with specific version being produced just once."""
         package_tuple = state.resolved_dependencies.get(self.configuration["package_name"])
