@@ -17,18 +17,22 @@
 
 """Wrap units implemented in adviser."""
 
-from .mkl_threads import MKLThreadsWrap
 from .no_onservation import NoObservationWrap
-from .no_semantic_interposition import NoSemanticInterpositionWrap
-from .intel_tensorflow import IntelTensorFlowWrap
+from .python import NoSemanticInterpositionWrap
+from .tensorflow import TensorFlow23DictSummary
+from .tensorflow import TensorFlow23Accuracy
+from .tensorflow import IntelTensorFlowWrap
+from .tensorflow import MKLThreadsWrap
 
 
 # Relative ordering of units is relevant, as the order specifies order
 # in which the asked to be registered - any dependencies between them
 # can be mentioned here.
 __all__ = [
-    "MKLThreadsWrap",
     "NoObservationWrap",
     "NoSemanticInterpositionWrap",
+    "TensorFlow23DictSummary",
+    "TensorFlow23Accuracy",
     "IntelTensorFlowWrap",
+    "MKLThreadsWrap",
 ]
